######################################
### Lab2: Sample size calculations ###
######################################

# 1. Estimation of sample size for mean estimation
# Find the smallest sample size ensuring an error of estimation within d = 0.5
# of the true mean with 95% confidence using the formula for sample size calculation
(2 * qnorm(0.975))^2

# Round up 'n' to the next highest integer for practical application
ceiling((2 * qnorm(0.975))^2)


# 2. Sample size calculation for a Normal mean hypothesis test
# Define hypothesis parameters:
# H0: mu <= 3 vs H1: mu = 4, alpha = 0.05, beta = 0.1, and sigma = 2
library(pwr)

# Review the help file for 'pwr.t.test'
?pwr.t.test

# Calculate the sample size needed for a one-sample t-test
pwr.t.test(d = (4 - 3) / 2, sig.level = 0.05, power = 0.90, 
           alternative = "greater", type = "one.sample")

# To determine the actual power of the test, given a sample size
pwr.t.test(d = (4 - 3) / 2, sig.level = 0.05, n = 36, 
           alternative = "greater", type = "one.sample")



# 3. Phase II situation: H0:p = 0.15 versus HA:p = 0.40,  = 0.1 and  = 0.2
# (i) Employing an exact binomial test
# Check documentation for the 'ph2single' function
?ph2single

# Determine sample size for a Phase II trial using an exact binomial test
ph2single(pu = 0.15, pa = 0.40, ep1 = 0.10, ep2 = 0.20, nsoln = 5)


# (ii) Normalizing the distribution of the test statistic using arcsine transformation of proportion
# Review the documentation for 'pwr.p.test'
?pwr.p.test

# Calculate sample size for a test statistic with arcsine transformation
pwr.p.test(h = ES.h(0.40, 0.15), sig.level = 0.10, 
           power = 0.80, alternative = "greater")

# 4. Verification of calculations from Table 11.11 in Piantadosi's textbook (2005, pp. 280)

# Create dataframes to replicate Table 11.11 calculations for different effect sizes and significance levels
table1 = data.frame(ES = seq(0.25, 1.50, by = 0.25),
                    beta = 0.10, "Alpha = 0.05" = NA, "Alpha = 0.10" = NA)
names(table1)[3:4] = c("Alpha = 0.05", "Alpha = 0.10")

# Calculate sample sizes for different effect sizes and significance levels (alpha = 0.05 and alpha = 0.10)
for (i in 1:6) {
  table1[i, 3] = ceiling(2 * pwr.t.test(d = table1[i, 1], sig.level = 0.05, 
                                        power = 0.90, alternative = "two.sided",
                                        type = "two.sample")$n)
  table1[i, 4] = ceiling(2 * pwr.t.test(d = table1[i, 1], sig.level = 0.10, 
                                        power = 0.90, alternative = "two.sided",
                                        type = "two.sample")$n)
}
table1

table2 = data.frame(ES = seq(0.25, 1.50, by = 0.25),
                    beta = 0.20, "Alpha = 0.05" = NA, "Alpha = 0.10" = NA)
names(table2)[3:4] = c("Alpha = 0.05", "Alpha = 0.10")

# Calculate sample sizes for different effect sizes and significance levels (alpha = 0.05 and alpha = 0.10)
for (i in 1:6) {
  table2[i, 3] = ceiling(2 * pwr.t.test(d = table2[i, 1], sig.level = 0.05, 
                                        power = 0.80, alternative = "two.sided",
                                        type = "two.sample")$n)
  table2[i, 4] = ceiling(2 * pwr.t.test(d = table2[i, 1], sig.level = 0.10, 
                                        power = 0.80, alternative = "two.sided",
                                        type = "two.sample")$n)
}
table2

# 5. Unequal group allocation using normal quantiles
# Load the 'gsDesign' library and check 'nNormal' function details
library(gsDesign)
?nNormal

# Create a dataframe to store sample sizes for different allocation ratios
ar = data.frame(ratio = 1 / 1:10, n = NA)

# Calculate sample sizes for varying allocation ratios using 'nNormal'
for (i in 1:10) {
  ar$n[i] = nNormal(
    delta1 = 0.5, delta0 = 0, sd = 1, sd2 = 1,
    alpha = 0.05, beta = 0.10,
    sided = 2, ratio = ar$ratio[i]
  )
}
ar

# Set the working directory for saving graphs
setwd("C:/Clinical trials/R labs/lab1/graphs")

# Generate a plot illustrating total sample size against ratio of sample sizes
pdf("allocation_ratio.pdf", height = 5.5, width = 5.5)
plot(ceiling(n) ~ I(1 / ratio), data = ar, xlim = c(1, 10), ylim = c(0, 550),
     xlab = "Ratio of sample sizes",
     ylab = "Total sample size", type = "l", xaxt = "n")
axis(1, at = 1:10)
dev.off()


# 6. Sample size calculation for survival data
# Check documentation for the 'nSurv' function
?nSurv

# Define median survival times for standard therapy
m0 = 15
lambda0 = -log(0.5) / m0

# Define median survival times for experimental therapy
m1 = 25
lambda1 = -log(0.5) / m1

# Calculate the ratio of survival rates and hazard ratios for exponential distribution
# Examine the properties between the two therapies
c(m1 / m0, lambda0 / lambda1)

# Assess sample size required for a survival trial

# Scenario 1: With specified accrual and follow-up periods, solving for accrual rate
nSurv(
  lambdaC = lambda0, hr = lambda1 / lambda0, hr0 = 1,
  R = 55, minfup = 10, T = 65,
  ratio = 1, alpha = 0.05, beta = 0.20, sided = 2
)

# Scenarios 2: Accounting for dropout rates
nSurv(
  lambdaC = lambda0, hr = lambda1 / lambda0, hr0 = 1,
  R = 55, minfup = 10, T = 65, eta = 0.009, etaE = 0.009,
  ratio = 1, alpha = 0.05, beta = 0.20, sided = 2
)

# Scenarios 3: Accounting for longer follow-up
nSurv(
  lambdaC = lambda0, hr = lambda1 / lambda0, hr0 = 1,
  R = 55, minfup = 100, T = 155,
  ratio = 1, alpha = 0.05, beta = 0.20, sided = 2
)

# Scenarios 4: Accounting for patient recruitment rate
nSurv(
  lambdaC = lambda0, hr = lambda1 / lambda0, hr0 = 1,
  gamma = 5, R = 35,
  ratio = 1, alpha = 0.05, beta = 0.20, sided = 2
)

# Investigate sample size variation based on follow-up duration

# Generate data on sample size and accrual rate for different follow-up periods
fp = seq(5, 100, by = 0.5)
n = rep(NA, length(fp))
ar = rep(NA, length(fp))

for (i in 1:length(fp)) {
  x = nSurv(
    lambdaC = lambda0, hr = lambda1 / lambda0, hr0 = 1,
    R = 5, minfup = fp[i], T = 5 + fp[i],
    ratio = 1, alpha = 0.05, beta = 0.20, sided = 2
  )
  n[i] = x$n
  ar[i] = x$gamma
}

# Save results and plot the relationship between follow-up duration and sample size
setwd("C:/Clinical trials/R labs/lab2/graphs")
pdf("sample_sizeS.pdf", height = 5.5, width = 5.5)
plot(
  fp, n, ylim = c(80, 510), yaxt = "n", ylab = "sample size",
  xlab = "Follow up (months)", type = "l"
)
axis(2, at = c(100, 120, 200, 300, 400, 500), las = 1)
abline(h = 119, col = "red")
text(18, 130, "Expected events")
dev.off()